#include "mainwidget.h"

MainWidget::MainWidget()
{
    Assimp::Importer importer;
    const aiScene* scene = importer.ReadFile("",
                                             aiProcess_CalcTangentSpace |
                                             aiProcess_Triangulate |
                                             aiProcess_JoinIdenticalVertices |
                                             aiProcess_SortByPType);

    if( !scene)
    {
        qDebug()<<importer.GetErrorString();
    }else {
        //DoTheSceneProcessing( scene);
    }
}

QSize MainWidget::minimumSizeHint() const
{
    return QSize(50, 50);
}

QSize MainWidget::sizeHint() const
{
    return QSize(1280, 720);
}


void MainWidget::initializeGL(){
    bool glFunctionsOK = initializeOpenGLFunctions();
    Q_ASSERT(glFunctionsOK);
    if(!glFunctionsOK)
        exit(4);

    qDebug()<<"Version"<<this->format().version();

    glClearColor(0.0, 0.0, 0.0, 0.0);
    glEnable(GL_DEPTH_TEST);
    glDisable(GL_CULL_FACE);

    // Shader
    animationShader = initShader(QLatin1String(":/animate.frag"),QLatin1String(":/animate.vert"));

    m_view = QMatrix4x4();
    m_view.lookAt(QVector3D(2.0,2.0,4.0),QVector3D(0.0,0.0,0.0),QVector3D(0.0,1.0,0.0));

    //temp
    static const GLfloat g_vertex_buffer_data[] = {
        -1.0f,-1.0f,-1.0f,
        -1.0f,-1.0f, 1.0f,
        -1.0f, 1.0f, 1.0f,
        1.0f, 1.0f,-1.0f,
        -1.0f,-1.0f,-1.0f,
        -1.0f, 1.0f,-1.0f,
        1.0f,-1.0f, 1.0f,
        -1.0f,-1.0f,-1.0f,
        1.0f,-1.0f,-1.0f,
        1.0f, 1.0f,-1.0f,
        1.0f,-1.0f,-1.0f,
        -1.0f,-1.0f,-1.0f,
        -1.0f,-1.0f,-1.0f,
        -1.0f, 1.0f, 1.0f,
        -1.0f, 1.0f,-1.0f,
        1.0f,-1.0f, 1.0f,
        -1.0f,-1.0f, 1.0f,
        -1.0f,-1.0f,-1.0f,
        -1.0f, 1.0f, 1.0f,
        -1.0f,-1.0f, 1.0f,
        1.0f,-1.0f, 1.0f,
        1.0f, 1.0f, 1.0f,
        1.0f,-1.0f,-1.0f,
        1.0f, 1.0f,-1.0f,
        1.0f,-1.0f,-1.0f,
        1.0f, 1.0f, 1.0f,
        1.0f,-1.0f, 1.0f,
        1.0f, 1.0f, 1.0f,
        1.0f, 1.0f,-1.0f,
        -1.0f, 1.0f,-1.0f,
        1.0f, 1.0f, 1.0f,
        -1.0f, 1.0f,-1.0f,
        -1.0f, 1.0f, 1.0f,
        1.0f, 1.0f, 1.0f,
        -1.0f, 1.0f, 1.0f,
        1.0f,-1.0f, 1.0f
    };



    glGenBuffers(1, &vertexbuffer);
    glBindBuffer(GL_ARRAY_BUFFER, vertexbuffer);
    glBufferData(GL_ARRAY_BUFFER, sizeof(g_vertex_buffer_data), g_vertex_buffer_data, GL_STATIC_DRAW);
}

QOpenGLShaderProgram* MainWidget::initShader(QString fragSource, QString vertSource){

    QOpenGLShader *vert = new QOpenGLShader(QOpenGLShader::Vertex);
    if(!vert->compileSourceFile(vertSource)){
        qCritical()<< "Fragment Shader"<<vertSource<<"failed"<< vert->log();
        exit(5);
    }

    QOpenGLShader *frag = new QOpenGLShader(QOpenGLShader::Fragment);
    if(!frag->compileSourceFile(fragSource)){
        qCritical()<< "Fragment Shader"<<fragSource<<"failed"<< frag->log();
        exit(5);
    }

    QOpenGLShaderProgram* shader = new QOpenGLShaderProgram();
    shader->addShader(vert);
    shader->addShader(frag);
    if(!shader->link()){
        qCritical()<< "Linking shader failed:"<<shader->log();
        exit(5);
    }

    return shader;
}


void MainWidget::paintGL(){
    glViewport(0,0,width(),height());
    qDebug()<<"Format"<<this->format().version();

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    animationShader->bind();
    QMatrix4x4 VP = m_projection*m_view;
    //QMatrix4x4 VP = m_view*m_projection;


    animationShader->setUniformValue("VP",VP);

    glEnableVertexAttribArray(positionIndex);

    glBindBuffer(GL_ARRAY_BUFFER, vertexbuffer);
    glVertexAttribPointer(positionIndex, 3, GL_FLOAT, GL_FALSE, 0,(void*)0 );

    glDrawArrays(GL_TRIANGLES, 0, 12*3);

    glDisableVertexAttribArray(positionIndex);

    animationShader->release();
}

void MainWidget::resizeGL(int width, int height){
    qDebug()<<"Resize"<<width<<height;

    m_projection = QMatrix4x4();
    m_projection.perspective(45.0,1.0*width/height,0.01,100.0);


}

